#!/bin/bash
#
# build_postgresql.sh

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <p279|p2710|p2711> [tar]"
   echo "        p279 -> python v2_7_9"
   echo "        p2710 -> python v2_7_10"
   echo "        p2711 -> python v2_7_11"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
        
    PRODUCTS=${product_dir}:${PRODUCTS}

    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${basequal}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    sourcedir=${product_dir}/${package}/${pkgver}/source
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${basequal}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
maketar=${3}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=postgresql
origpkgver=v9_3_12
pkgver=${origpkgver}
psycopg2dotver=2.6.1
ssibuildshims_version=v0_09_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=postgresql-${pkgdotver}

get_this_dir

get_ssibuildshims

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${basequal}
    exit 0
fi

echo "building ${package} for ${OS}-${plat} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${basequal} in
p279)  ;;
p2710)  ;;
p2711)  ;;
*) 
   echo "ERROR: unrecognized qualifier ${basequal}"
   usage
   exit 1
esac

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${basequal} || die "ERROR: fake declare failed"

setup ${package} ${pkgver} -q ${basequal} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed."

## the build requires python
if [ -z ${PYTHONHOME} ]
then
   echo "ERROR: PYTHONHOME is undefined"
   echo "       something is very wrong"
   exit 1
else 
   ls ${PYTHONHOME}/lib
fi


ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${sourcedir}
tar xf ${tardir}/${pkgtarname}.tar.bz2

( cd "${pkgtarname}" && patch < "${patchdir}/configure-python.patch"  || exit 1) || \
 die "ERROR: unable to apply configure patch."

cd ${blddir}
${sourcedir}/${pkgtarname}/configure --prefix=${pkgdir} --with-krb5 --with-gssapi --with-python || \
 die "ERROR: configure failed."

make -j ${ncore} || die "ERROR: make failed."

# install only the client: 
make -C src/bin install && \
make -C src/include install && \
make -C src/interfaces install && \
make -C src/pl install || die "ERROR: make install failed."

set +x

# Building documentation from source is problematic on many systems.
# make -C doc install

# now build psycopg2
echo
echo "begin psycopg2 build"
echo
cd ${sourcedir}
tar xf ${tardir}/psycopg2-${psycopg2dotver}.tar.gz

set -x

cd ${sourcedir}/psycopg2-${psycopg2dotver}
echo
python setup.py build_ext --pg-config  ${POSTGRESQL_FQ_DIR}/bin/pg_config install --prefix ${pkgdir} || exit 1
echo

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${basequal} || die "ERROR: declare failed"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${basequal} -z ${product_dir}:${PRODUCTS}
echo "${package} is installed at ${PGSQL_DIR}"
echo "PYTHONPATH is ${PYTHONPATH}"

# NOTE: to test psycopg2
# setup ${package} ${pkgver} -q ${basequal} -z ${product_dir}:${PRODUCTS}
# python
# >> import psycopg2

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver}  ${basequal}
fi

exit 0
